﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace CSV2SHP
{
    public partial class Form1 : Form
    {
        private ArrayList fieldInfoList = new ArrayList();

        public Form1()
        {
            InitializeComponent();
        }

        private List<String> SeperateStringWithComma(String value)
        {
            bool inQuotes = false;
            char delim = ',';
            List<String> strings = new List<String>();

            StringBuilder sb = new StringBuilder();
            foreach (char c in value)
            {
                if (c == '\'' || c == '"')
                {
                    if (!inQuotes)
                        inQuotes = true;
                    else
                        inQuotes = false;
                }
                if (c == delim)
                {
                    if (!inQuotes)
                    {
                        strings.Add(sb.Replace("'", string.Empty).Replace("\"", string.Empty).ToString());
                        sb.Remove(0, sb.Length);
                    }
                    else
                    {
                        sb.Append(c);
                    }
                }
                else
                {
                    sb.Append(c);
                }
            }

            strings.Add(sb.Replace("'", string.Empty).Replace("\"", string.Empty).ToString());

            return strings;
        }

        private void fillFieldList()
        {
            String FileName = tbCSV.Text;
            System.IO.StreamReader file = new System.IO.StreamReader(FileName);
            String line;
            bool firstLine = true;
            fieldInfoList.Clear();
            lbFields.Items.Clear();
            while ((line = file.ReadLine()) != null)
            {
                if (firstLine)
                {
                    firstLine = false;
                    List<String> values = SeperateStringWithComma(line);

                    lbFields.Items.Clear();

                    int cntValues = values.Count;
                    for(int iValue=0; iValue<cntValues; iValue++)
                    {
                        String fieldName = values[iValue];
                        lbFields.Items.Add(fieldName);
                        FieldInfo fi = new FieldInfo(fieldName, fieldInfoList);
                    }

                    return;
                }
            }

            file.Close();
        }

        private void btnCSV_Click(object sender, EventArgs e)
        {
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbCSV.Text = openFileDialog.FileName;

                fillFieldList();
            }
        }

        private void btnSHP_Click(object sender, EventArgs e)
        {
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbSHP.Text = saveFileDialog.FileName;
            }
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            XrMapLib.FieldSetDefine FSD = new XrMapLib.FieldSetDefine();
            int cntFields = fieldInfoList.Count;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                FieldInfo fi = fieldInfoList[iField] as FieldInfo;

                if (!fi.GetExport() || fi.GetHorizontalCoord() || fi.GetVerticalCoord()) continue; 

                XrMapLib.XrFieldTypeEnum type = fi.GetFieldType();
                String FieldName = fi.GetFieldName();
                if (type == XrMapLib.XrFieldTypeEnum.XrString)
                {
                    FSD.AddStringField(FieldName, 100);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrDouble)
                {
                    FSD.AddFloatField(FieldName, 16, 6);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrInteger)
                {
                    FSD.AddIntegerField(FieldName, 10);
                }
            }

            XrMapLib.Tool tool = new XrMapLib.Tool();
            XrMapLib.EsriShpFileHandler SFH = tool.CreateEsriShpFileHandler(tbSHP.Text, XrMapLib.XrShapeTypeEnum.XrPoint, FSD);
            if (SFH == null)
            {
                MessageBox.Show("Error: Creating shp file");
                return;
            }

            String FileName = tbCSV.Text;
            System.IO.StreamReader file = new System.IO.StreamReader(FileName);
            String line;
            bool firstLine = true;

            while ((line = file.ReadLine()) != null)
            {
                if (firstLine)
                {
                    firstLine = false;
                    continue;
                }

                List<String> values = SeperateStringWithComma(line);
                XrMapLib.ValueList VL = new XrMapLib.ValueList();
                double X=0, Y=0;
                for (int iField = 0; iField < cntFields; ++iField)
                {
                    FieldInfo fi = fieldInfoList[iField] as FieldInfo;

                    if (!fi.GetExport()) continue;
                    
                    String Value = values[iField];

                    if (fi.GetHorizontalCoord())
                    {
                        X = Double.Parse(Value);
                        continue;
                    }

                    if(fi.GetVerticalCoord())
                    {
                        Y = Double.Parse(Value);
                        continue;
                    }

                    XrMapLib.XrFieldTypeEnum type = fi.GetFieldType();
                    String FieldName = fi.GetFieldName();
                    
                    if (type == XrMapLib.XrFieldTypeEnum.XrString)
                    {
                        VL.AddValueByString(Value);
                    }
                    else if (type == XrMapLib.XrFieldTypeEnum.XrDouble)
                    {
                        VL.AddValueByFloat(Double.Parse(Value), 6);
                    }
                    else if (type == XrMapLib.XrFieldTypeEnum.XrInteger)
                    {
                        VL.AddValueByInteger(Int32.Parse(Value));
                    }
                }

                if (!SFH.AddPoint(X, Y, VL))
                {
                    MessageBox.Show("Error: Writing SHP");
                    return;
                }

            }

            file.Close();

            MessageBox.Show("Completed");
        }

        private void lbFields_SelectedIndexChanged(object sender, EventArgs e)
        {
            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            
            cbExport.Checked = fi.GetExport();
            rbHCoord.Checked = fi.GetHorizontalCoord();
            rbVCoord.Checked = fi.GetVerticalCoord();
            
            XrMapLib.XrFieldTypeEnum type = fi.GetFieldType();
            if (type == XrMapLib.XrFieldTypeEnum.XrString) rbString.Checked = true;
            else if (type == XrMapLib.XrFieldTypeEnum.XrDouble) rbFloat.Checked = true;
            else if (type == XrMapLib.XrFieldTypeEnum.XrInteger) rbInteger.Checked = true;
        }

        private void cbExport_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetExport(cbExport.Checked);

            if (!cbExport.Checked)
            {
                rbHCoord.Checked = false;
                rbVCoord.Checked = false;
            }
        }

        private void rbHCoord_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetHorizontalCoord(rbHCoord.Checked);
        }

        private void rbVCoord_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetVerticalCoord(rbVCoord.Checked);
        }

        private void rbString_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetFieldType(XrMapLib.XrFieldTypeEnum.XrString);
        }

        private void rbInteger_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetFieldType(XrMapLib.XrFieldTypeEnum.XrInteger);
        }

        private void rbFloat_CheckedChanged(object sender, EventArgs e)
        {
            if (lbFields.SelectedIndex == -1) return;

            FieldInfo fi = fieldInfoList[lbFields.SelectedIndex] as FieldInfo;
            fi.SetFieldType(XrMapLib.XrFieldTypeEnum.XrDouble);
        }
    }
}
